@@ -8,6 +8,7 @@ import re
 import time
 import textwrap
 import threading
+import operator
 from datetime import datetime
 from datetime import timedelta, tzinfo
 from boto3.s3.transfer import S3Transfer
@@ -78,6 +79,7 @@ class AWSCloudFormation(AWSSession):
             self.easy_service = False
 
             self.on_failure = kwargs.get('on_failure', 'DELETE')
+            self.role_arn = kwargs.get('role_arn', None)
 
             if 'template' in kwargs and type(kwargs['template']) == str:
                 self.template = kwargs['template']
@@ -196,6 +198,7 @@ class AWSCloudFormation(AWSSession):
 
     def print_outputs(self, **kwargs):
         stack_outputs = self.outputs(**kwargs)
+        stack_outputs.sort(key=operator.itemgetter('OutputKey'))
 
         if len(stack_outputs) == 0:
             print("Stack {} don't have any outputs".format(self.stack_name))
@@ -306,7 +309,7 @@ class AWSCloudFormation(AWSSession):
                 return output['OutputValue']
 
         default = kwargs.get('default', None)
-        if default:
+        if default is not None:
             return default
 
         print("Can't find output parameter {} in stack {} under {} profile".format(
@@ -363,23 +366,27 @@ class AWSCloudFormation(AWSSession):
 
         cloudformation = self.client('cloudformation')
 
-        stack_name = self.stack_name
-        if 'stack_name' in kwargs:
-            stack_name = kwargs.get('stack_name')
+        stack_name = kwargs.get('stack_name', self.stack_name)
+        role_arn = kwargs.get('role_arn', self.role_arn)
 
-        template_url = "https://s3.amazonaws.com/%s/%s" % (
-            self.s3_bucket, self.s3_key)
-        print("Creating stack {}".format(stack_name))
+        template_url = "https://s3.amazonaws.com/{}/{}".format(self.s3_bucket, self.s3_key)
+
+        role_arn_txt = " using role_arn={}".format(role_arn) if role_arn else ""
+        print("Creating stack {stack_name} in {profile_name}{role_arn_txt}".format(stack_name=stack_name, profile_name=self.profile_name, role_arn_txt=role_arn_txt))
         stack_id = None
         parameters = self._join_parameters(self.parameters, kwargs.get('parameters', None))
         try:
-            resp = cloudformation.create_stack(
-                StackName=stack_name,
-                TemplateURL=template_url,
-                Capabilities=['CAPABILITY_NAMED_IAM'],
-                OnFailure=self.on_failure,
-                Parameters=parameters
-            )
+            args = {
+                'StackName': stack_name,
+                'TemplateURL': template_url,
+                'Capabilities': ['CAPABILITY_NAMED_IAM'],
+                'Parameters': parameters,
+                'OnFailure': self.on_failure
+            }
+            if role_arn:
+                args['RoleARN'] = role_arn
+
+            resp = cloudformation.create_stack(**args)
             resp = cloudformation.describe_stacks(
                 StackName=resp['StackId']
             )
@@ -387,7 +394,7 @@ class AWSCloudFormation(AWSSession):
             timestamp = resp['Stacks'][0]['CreationTime']
         except botocore.exceptions.ParamValidationError as error:
             if str(error).startswith("Parameter validation failed:\nInvalid type for parameter"):
-                res = re.search((r'Invalid type for parameter Parameters\[(\d)\].ParameterValue, '
+                res = re.search((r'Invalid type for parameter Parameters\[(\d+)\].ParameterValue, '
                                 r'value: None, type: (.+), valid types: (.+)'), str(error))
                 param_idx = int(res.group(1))
                 param_type_asis = res.group(2)
@@ -425,22 +432,26 @@ class AWSCloudFormation(AWSSession):
 
         cloudformation = self.client('cloudformation')
 
-        stack_name = self.stack_name
-        if 'stack_name' in kwargs:
-            stack_name = kwargs.get('stack_name')
+        stack_name = kwargs.get('stack_name', self.stack_name)
+        role_arn = kwargs.get('role_arn', self.role_arn)
 
-        template_url = "https://s3.amazonaws.com/%s/%s" % (
-            self.s3_bucket, self.s3_key)
-        print("Updating stack {}".format(stack_name))
+        template_url = "https://s3.amazonaws.com/{}/{}".format(self.s3_bucket, self.s3_key)
+
+        role_arn_txt = " using role_arn={}".format(role_arn) if role_arn else ""
+        print("Updating stack {stack_name} in {profile_name}{role_arn_txt}".format(stack_name=stack_name, profile_name=self.profile_name, role_arn_txt=role_arn_txt))
         parameters = self._join_parameters(self.parameters, kwargs.get('parameters', None))
         timestamp = datetime.now(utc)
         try:
-            resp = cloudformation.update_stack(
-                StackName=stack_name,
-                TemplateURL=template_url,
-                Capabilities=['CAPABILITY_NAMED_IAM'],
-                Parameters=parameters
-            )
+            args = {
+                'StackName': stack_name,
+                'TemplateURL': template_url,
+                'Capabilities': ['CAPABILITY_NAMED_IAM'],
+                'Parameters': parameters
+            }
+            if role_arn:
+                args['RoleARN'] = role_arn
+
+            resp = cloudformation.update_stack(**args)
             respZ = cloudformation.describe_stacks(
                 StackName=resp['StackId']
             )
@@ -483,21 +494,25 @@ class AWSCloudFormation(AWSSession):
     def delete(self, **kwargs):
         cloudformation = self.client('cloudformation')
 
-        stack_name = self.stack_name
-        if 'stack_name' in kwargs:
-            stack_name = kwargs.get('stack_name')
+        stack_name = kwargs.get('stack_name', self.stack_name)
+        role_arn = kwargs.get('role_arn', self.role_arn)
 
         resp = cloudformation.describe_stacks(StackName=stack_name)
 
-        print('Deleting stack {}'.format(stack_name))
+        role_arn_txt = " using role_arn={}".format(role_arn) if role_arn else ""
+        print("Deleting stack {stack_name} in {profile_name}{role_arn_txt}".format(stack_name=stack_name, profile_name=self.profile_name, role_arn_txt=role_arn_txt))
         timestamp = datetime.now(utc)
-        cloudformation.delete_stack(
-            StackName=resp['Stacks'][0]['StackId']
-        )
+
+        args = {
+            'StackName': resp['Stacks'][0]['StackId']
+        }
+        if role_arn:
+            args['RoleARN'] = role_arn
+
+        cloudformation.delete_stack(**args)
         respZ = cloudformation.describe_stacks(
             StackName=resp['Stacks'][0]['StackId']
         )
-        print('A: {}; B: {}'.format(datetime.utcnow(), respZ['Stacks'][0]['DeletionTime']))
 
         self._print_events(resp['Stacks'][0]['StackId'], stack_name, timestamp)
 
@@ -666,6 +681,23 @@ class AWSCloudFormation(AWSSession):
 
         return
 
+    def dump_parameters(self, **kwargs):
+        print("Dumping input parameters to json")
+
+        parameters = self._join_parameters(self.parameters, kwargs.get('parameters', None))
+
+        result = {'Parameters': dict()}
+
+        for param in parameters:
+            result['Parameters'][param.get('ParameterKey')] = param.get('ParameterValue')
+
+        out_file = kwargs.get('out_file', None)
+        if out_file:
+            with open(out_file, 'w') as file:
+                file.write(json.dumps(result, indent=2))
+        else:
+            print(json.dumps(result, indent=2))
+
     def _upload(self):
         print("Uploading %s to s3://%s/%s" %
               (self.template, self.s3_bucket, self.s3_key))
